@@ -84,12 +84,15 @@ class CDx_Data():
         self.crosstab = self.get_crosstab()
 
     def __len__(self):
-        return len(self.cli)
+        return 0 if self.cli is None else len(self.cli)
 
     def __getitem__(self, n):
         return self.select_by_sample_ids([self.cli.sampleId.iloc[n]])
 
     def __sub__(self, cdx):
+        if self.cli is None and cdx.cli is None:
+            return CDx_Data()
+
         cli = pd.concat([self.cli, cdx.cli]).drop_duplicates(keep=False)
         mut = pd.concat([self.mut, cdx.mut]).drop_duplicates(keep=False)
         cnv = pd.concat([self.cnv, cdx.cnv]).drop_duplicates(keep=False)
@@ -97,7 +100,10 @@ class CDx_Data():
 
         return CDx_Data(cli_df=cli, mut_df=mut, cnv_df=cnv, sv_df=sv)
 
-    def __add__(self,cdx):
+    def __add__(self, cdx):
+        if self.cli is None and cdx.cli is None:
+            return CDx_Data()
+
         cli = pd.concat([self.cli, cdx.cli]).drop_duplicates()
         mut = pd.concat([self.mut, cdx.mut]).drop_duplicates()
         cnv = pd.concat([self.cnv, cdx.cnv]).drop_duplicates()
@@ -105,7 +111,6 @@ class CDx_Data():
 
         return CDx_Data(cli_df=cli, mut_df=mut, cnv_df=cnv, sv_df=sv)
 
-
     def from_PETA(self, token: str, json_str: str):
         """Retrieve CDx data from BGI-PETA database. 
 
@@ -126,7 +131,7 @@ class CDx_Data():
 
         # dedup for the same sampleId in different studyIds, discard the duplicated ones from all tables
         cli_original = self.cli
-        self.cli = self.cli.drop_duplicates(subset=['sampleId', 'studyId'])
+        self.cli = self.cli.drop_duplicates('sampleId')
         if (len(self.cli) < len(cli_original)):
             print('Duplicated sampleId exists, drop duplicates and go on')
 
@@ -135,13 +140,13 @@ class CDx_Data():
 
         self.sv = self.sv[self.sv.apply(
             lambda x: (x['Tumor_Sample_Barcode'], x['studyId']) in undup_tuple,
-            axis=1)]
+            axis=1)].drop_duplicates()
         self.cnv = self.cnv[self.cnv.apply(
             lambda x: (x['Tumor_Sample_Barcode'], x['studyId']) in undup_tuple,
-            axis=1)]
+            axis=1)].drop_duplicates()
         self.mut = self.mut[self.mut.apply(
             lambda x: (x['Tumor_Sample_Barcode'], x['studyId']) in undup_tuple,
-            axis=1)]
+            axis=1)].drop_duplicates()
 
         # time series
         self.cli = self._infer_datetime_columns()
@@ -786,6 +791,9 @@ class CDx_Data():
         Returns:
             Union[float,pd.Series]: A pd.Series when groupby options passed, a float value when not.
         """
+        # empty CDx
+        if len(self)==0:
+            return pd.Series([], dtype='float64')
 
         crosstab = self.crosstab.reindex(index=variant_type_to_observe,
                                          level=0)
@@ -808,6 +816,11 @@ class CDx_Data():
         else:
             test_posi_rate = self._crosstab_to_positive_rate(crosstab)
 
+        # default to retrun a empty DataFrame which can cause problems
+        if (not isinstance(test_posi_rate,
+                           float)) and len(test_posi_rate) == 0:
+            test_posi_rate = pd.Series([], dtype='float64')
+
         return test_posi_rate
 
     def _crosstab_to_positive_rate(self, df: pd.DataFrame):
